function [apogee, maxAccel] = quickApogeeOnly(rocket, environment, wind, settings, wrapper)
arguments
    rocket      Rocket
    environment Environment
    wind        WindCustom
    settings    Settings
    wrapper     = []; % DataWrapper (empty by default)
end
%{
quickApogeeOnly - This function tests the fins simulating the ascent

INPUTS:
- settings,    struct (motor, CoeffsE, CoeffsF, para, ode, stoch, prob, wind), rocket data structure
                   

OUTPUTS:
- apogee,      double [1, 1], maximum altitude reached by the rocket [m];
- maxAccel,    double [1, 1], maximum acceleration of the rocket [m/s^2];
- vExit,       double [1, 1], launchpad exit velocity [m/s];

CALLED FUNCTIONS: windConstGenerator.

REVISIONS:
- 0 14/10/2019, Release, Matteo Pozzoli

- 1 21/10/2020, Second version, Adriano Filippo Inno

- 2 03/10/2021, Update, Davide Rosato
                Multiple and smooth airbrakes opening added


Copyright © 2021, Skyward Experimental Rocketry, AFD department
All rights reserved

SPDX-License-Identifier: GPL-3.0-or-later

%}

%% ERROR CHECKING
if not(rocket.airbrakes.enabled) && length(rocket.airbrakes.extension) > 1
    error('To simulate different airbrakes opening, please set to true airbrakes.enabled in rocketConfig.m');
end

if rocket.airbrakes.enabled && length(rocket.airbrakes.extension) > 1 && length(rocket.airbrakes.deltaTime) < length(rocket.airbrakes.extension) - 2
    error('In airbrakes smooth opening simulations, AB configuration usage time vector must be at least of length length(pCOntrol)-2, check config.m')
end

%% DATA PREPARATION
if isempty(wrapper)
    options = odeset(settings.ode.optAscent);
else
    options = odeset(settings.ode.optAscent, 'OutputFcn', @storeData);
end
tf = settings.ode.finalTime;

%% INITIAL CONDITIONS
%%% Attitude
Q0 = angleToQuat(environment.phi, environment.omega, 180*pi/180)';

%%% State
X0 = [0; 0; 0];
V0 = [0; 0; 0];
W0 = [0; 0; 0];

Y0a = [X0; V0; W0; Q0; 0];

%% ASCENT
t0 = 0;
solution = ode113(@ballistic, [t0, tf], Y0a, options, ...
    rocket, environment, wind, [], 0, wrapper);
ascent = getOdeFcn(solution);

%% CALCULATE OUTPUT QUANTITIES
apogee = -ascent.state.Y(3, end);

if nargout == 2
    accelerations = ascent.accelerations.body;
    absA = vecnorm(accelerations, 2, 1);
    [maxAccel] = max(absA)/9.81;
end
end